

import os
import subprocess
import sys
import re


E_SUCCESS = 0
E_UNSUPPORTED = 1
E_NOTFORMAT = 2
E_CURRUPTED = 3
E_STD_LIBRARY = 4
E_SYSTEM = 5
E_UNHANDLED = 6
E_NOT_IMPLEMENTED = 7
E_CONFIGURE_TOOL = 8

_TOOL_NAME = 'readpe.exe'
_TOOL_NAME_PATTERN = 'readpe(.*).exe'

class ReadpeException(Exception):
	def __init__(self, toolResult):
		Exception.__init__(self)
		self._toolResult = toolResult

	def __str__(self):
		return 'Exit code: %d' % self.error_code()

	def error_code(self):
		return self._toolResult['retcode']

	def stdout(self):
		return self._toolResult['stdout']

	def stderr(self):
		return self._toolResult['stderr']

class ScriptException(Exception):
	def __init__(self, errtext):
		Exception.__init__(self)
		self._errtext = errtext

	def __str__(self):
		return 'ReadPE Script error. ' + self._errtext

class ReadPE:
	def __init__(self):
		self._apppath = 'readpe'
		self._args = '--nologo --all'
		self._toolResult = dict()

	def _check_srcfile(self, srcfile):
		if not os.path.exists(srcfile) :
			raise ScriptException('Source file not exists')

	def _check_results(self):
		if self._toolResult['retcode'] != E_SUCCESS :
			raise ReadpeException(self._toolResult)
		if not self._toolResult['stdout'] :
			raise ScriptException('STDOUT is empty')
		if not self._toolResult['stderr'] :
			raise ScriptException('STDERR is empty')

	def _run_process(self, args):
		proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		stdoutBin = proc.stdout.read()
		stderrBin = proc.stderr.read()
		proc.stdout.close()
		proc.stderr.close()
		proc.wait()
		self._toolResult['retcode'] = proc.returncode
		self._toolResult['stdout'] = stdoutBin.decode('ascii')
		self._toolResult['stderr'] = stderrBin.decode('ascii')
		self._check_results()

	def run(self, srcfile):
		self._check_srcfile(srcfile)
		try:
			args = '%s %s "%s"' % (self._apppath, self._args, srcfile)
			self._run_process(args)
		except UnicodeDecodeError as e:
			raise ScriptException('Connot decode ReadPE stdout or stderr')

	def _is_it_readpe(self, filename):
		nameRe =re.compile(_TOOL_NAME_PATTERN, re.IGNORECASE)
		return not nameRe.match(filename) is None

	def set_app_path(self, path):
		# Проверка от случайной передачи пути к файлу малвари
		curToolName = os.path.basename(path)
		if self._is_it_readpe(curToolName):
			self._apppath = path
		else:
			errText = 'Expected tool name: "%s", Current name: "%s"' % (_TOOL_NAME, curToolName)
			raise ScriptException(errText)

	def set_command_args(self, args):
		self._args = args

	def stdout(self):
		return self._toolResult['stdout']

	def stderr(self):
		return self._toolResult['stderr']

def call_readpe_tool(filename, apppath='readpe.exe', args='--nologo --raw --export --imports --tls --detailed'):
	rpe = ReadPE()
	rpe.set_app_path(apppath)
	rpe.set_command_args(args)
	rpe.run(filename)
	return dict( stdout=rpe.stdout(), stderr=rpe.stderr() )

if __name__ == '__main__' :
	try:
		toolFile = sys.argv[1]
		testVector = sys.argv[2]
		results = call_readpe_tool(testVector, toolFile)
		print('[DEBUG] [STDERR]\n%s' % results['stderr'], file=sys.stderr)
		print('[DEBUG] [STDOUT]\n%s' % results['stdout'], file=sys.stderr)
		print('\n', file=sys.stderr)
	except ReadpeException as e:
		print('ReadPE tool error. %s' % e, file=sys.stderr)
		sys.exit(1)
	except ScriptException as e:
		print(str(e), file=sys.stderr)
		sys.exit(1)
