
import os
import sys
import json
import hashlib

from . import readpe

__all__ = ['TestSampleException', 'TestSample']

class TestSampleException(Exception):
	def __init__(self, errTxt):
		Exception.__init__(self)
		self._errTxt = errTxt

	def __str__(self):
		return self._errTxt


def _new_filename(directory, filename, extension):
	name = os.path.basename(filename).split(os.extsep)[0] + '.' + extension
	return os.path.join(directory, name)

def _write_to_file(filename, content):
	with open(filename, 'wb') as file_:
		file_.write(content.encode() )

def _md5_of_file(filename):
	md5 = hashlib.md5()
	with open(filename, 'rb') as f :
		for chunk in iter(lambda: f.read(md5.block_size * 128), ''):
			if not chunk:
				break
			md5.update(chunk)
	return md5.hexdigest().upper()

def _md5_of_text(strContent):
	result = 'None'
	if strContent :
		contentBin = strContent.encode(encoding='ascii')
		md5 = hashlib.md5()
		md5.update(contentBin)
		result = md5.hexdigest().upper()
	return result

class TestSample():
	def __init__(self, filename, toolFile='readpe.exe', cmdlineArgs=''):
		self._toolFile = toolFile
		self._cmdlineArgs = cmdlineArgs
		self._descriptor = dict()
		self._descriptor['source-filename'] = filename
		self._descriptor['source-md5'] = _md5_of_file(filename)
		self._clean_results()

	def _clean_results(self):
		self._descriptor['retcode'] = readpe.E_SUCCESS
		self._descriptor['stdout-content'] = ''
		self._descriptor['stderr-content'] = ''

	def _call_readpe(self):
		self._clean_results()
		filename = self._descriptor['source-filename']
		try:
			if self._cmdlineArgs:
				toolResult = readpe.call_readpe_tool(filename, self._toolFile, self._cmdlineArgs)
			else:
				toolResult = readpe.call_readpe_tool(filename, self._toolFile)
			self._descriptor['stdout-content'] = toolResult['stdout']
			self._descriptor['stderr-content'] = toolResult['stderr']
		except readpe.ReadpeException as e:
			self._descriptor['retcode'] = e.error_code()
			self._descriptor['stdout-content'] = e.stdout()
			self._descriptor['stderr-content'] = e.stderr()
		except readpe.ScriptException as e:
			self._clean_results()
			errTxt = 'File: "%s", Error: %s' % (filename, str(e))
			raise TestSampleException(errTxt)

	def _sort_stdout(self):
		result = self._descriptor['stdout-content']
		try:
			if self._descriptor['stdout-content']:
				parsed = json.loads(self._descriptor['stdout-content'])
				result = json.dumps(parsed, indent='\t', sort_keys=True)
		except ValueError:
			pass
		return result

	def prepare(self):
		self._call_readpe()
		self._descriptor['stdout-md5'] = _md5_of_text(self._descriptor['stdout-content'])
		self._descriptor['stderr-md5'] = _md5_of_text(self._descriptor['stderr-content'])

	def get_descriptor(self):
		result = self._descriptor.copy()
		del(result['stdout-content'])
		del(result['stderr-content'])
		return result

	def write_result(self, directory, param):
		if directory and os.path.exists(directory):
			filename = _new_filename(directory, self._descriptor['source-filename'], param)
			content = self._sort_stdout() if param == 'stdout' else \
				self._descriptor['stderr-content']
			_write_to_file(filename, content)

	def write_results(self, directory):
		self.write_result(directory, 'stdout')
		self.write_result(directory, 'stderr')
