# Find file dublicates in directory
#
# Syntax usage:
#   c:\python33\python.exe dublicates.py -d <directory path> [-r | --recursive] [--no-remove]
# Example:
#  @echo off
#  @set PythonDir=c:\\python33\\
#  @set ScriptsDir=.
#  @set InputDir=%*
#  %PythonDir%\python.exe %ScriptsDir%dublicates.py ^
#    --directory %InputDir% --recursive --no-remove ^
#    1>filenames.txt 2>errors.txt


import os
import sys
import hashlib
import argparse
import traceback


E_SUCCESS=0
E_INCORRECT_DIRECTORY=1
E_UNKNOWN=2


def get_files_from_directory(directory, recursive=False):
	"""@param directory: directory path
	@param recursive: None or True, if True handle directory recursive
	@return: flat list of files inside directory according to recursive flag
	"""
	for dirPath, subDirs, filenames in os.walk(directory, True):  #os.walk() don't walk to symbolic links to subdirectories
		for fname in filenames:
			yield(os.path.join(dirPath, fname))
		if not recursive:  #if we don't want to handle directory recursive we must stop here
			break

def md5_of_file(filename):
	md5 = hashlib.md5()
	with open(filename, 'rb') as f :
		for chunk in iter(lambda: f.read(md5.block_size * 128), ''):
			if not chunk:
				break
			md5.update(chunk)
	return md5.hexdigest().upper()

def compute_md5_of_files(files):
	result = dict()
	for fname in files:
		md5 = md5_of_file(fname)
		if not md5 in result:
			result[md5] = []
		result[md5].append(fname)
	return result

def find_dublicates(md5_file_list):
	result = dict()
	for md5 in list(md5_file_list.keys()):
		are_there_dublicates = len(md5_file_list[md5]) > 1
		if are_there_dublicates:
			result[md5] = md5_file_list[md5]
	return result

def print_dublicates(dublicates):
	for md5 in dublicates:
		filenames = dublicates[md5]
		s = filenames[0] + ':\n'
		for fname in filenames[1:]:
			s += ('\t* %s\n' % fname)
		print(s)

def remove_dublicates(dublicates):
	for md5 in dublicates:
		filenames = dublicates[md5]
		for fname in filenames[1:]:
			try:
				os.remove(fname)
				print('removed: "%s"' % fname)
			except BaseException:
				print('Problem: ' + fname, file=sys.stderr)
				raise

def parse_cmdline(argv=None):
	if argv != None:
		sys.argv = argv
	progname = os.path.basename(sys.argv[0])
	parser = argparse.ArgumentParser(prog=progname, description='Find and remove dublicate files')
	parser.add_argument('-d', '--directory', help='source directory', dest='directory')
	parser.add_argument('-r', '--recursive', help='recursive mode', dest='recursive', action='store_true', default=False)
	parser.add_argument('--no-remove', help='no remove', dest='no_remove', action='store_true', default=False)
	result = vars(parser.parse_args())
	return result

def is_directory_correct(directory):
	result = True
	directory_=os.path.abspath(directory) + os.sep
	if not (os.path.isdir(directory_) and os.path.exists(directory_) ):
		print('Invalid directory path', file=sys.stderr)
		result = False
	return result

def main(argv=None):
	result = E_SUCCESS
	try:
		args = parse_cmdline(argv)
		if is_directory_correct(args['directory']):
			files = get_files_from_directory(args['directory'], args['recursive'])
			md5_file_list = compute_md5_of_files(files)
			dublicates = find_dublicates(md5_file_list)
			if args['no_remove']:
				print_dublicates(dublicates)
			else:
				remove_dublicates(dublicates)
		else:
			print('Error: Incorrect directory argument', file=sys.stderr)
			result = E_INCORRECT_DIRECTORY
	except BaseException as e:
		print(traceback.format_exc(), file=sys.stderr)
		result = E_UNKNOWN
	return result

if __name__ == '__main__':
	sys.exit(main())
